#!/usr/bin/env python
# coding: utf-8

"""Swagger generic API client. This client handles the client-
server communication, and is invariant across implementations. Specifics of
the methods and models for each application are generated from the Swagger
templates."""

from __future__ import absolute_import
from .rest import RESTClient
from . import models

import os
import re
import datetime
import mimetypes

# python 2 and python 3 compatibility library
from six import iteritems, text_type

try:
    # for python3
    from urllib.parse import quote
except ImportError:
    # for python2
    from urllib import quote


class ApiClient(object):
    """
    Generic API client for Swagger client library builds

    :type configuration: ``ApiConfiguration``
    :param configuration: The ApiConfiguration for this client
    """
    def __init__(self, configuration):
        self.configuration = configuration
        self.default_headers = {}
        self.host = configuration.host
        # Set default User-Agent.
        self.user_agent = 'Python-Swagger'

    @property
    def user_agent(self):
        return self.default_headers['User-Agent']

    @user_agent.setter
    def user_agent(self, value):
        self.default_headers['User-Agent'] = value

    def call_api(
        self, resource_path, method, path_params=None, query_params=None,
        header_params=None, body=None, post_params=None, files=None,
        response=None, auth_settings=['default']
    ):

        # headers parameters
        header_params = header_params or {}
        header_params.update(self.default_headers)
        header_params = self.sanitize_for_serialization(header_params)

        # path parameters
        if path_params:
            path_params = self.sanitize_for_serialization(path_params)
            for k, v in iteritems(path_params):
                replacement = quote(str(self.to_path_value(v)))
                resource_path = resource_path.replace('{' + k + '}', replacement)

        # query parameters
        if query_params:
            query_params = self.sanitize_for_serialization(query_params)
            query_params = {k: self.to_path_value(v) for k, v in iteritems(query_params)}

        # post parameters
        if post_params:
            post_params = self.prepare_post_parameters(post_params, files)
            post_params = self.sanitize_for_serialization(post_params)

        # auth setting
        self.update_params_for_auth(header_params, query_params, auth_settings)

        # body
        if body:
            body = self.sanitize_for_serialization(body)

        # request url
        url = self.host + resource_path

        # perform request and return response
        response_data = self.request(
            method, url, query_params=query_params, headers=header_params,
            post_params=post_params, body=body)

        # deserialize response data
        if response:
            # return a tuple of models if the response is a list of objects
            if type(response_data) in [list, tuple]:
                return (
                    self.deserialize(model, response)
                    for model in response_data
                )
            return self.deserialize(response_data, response)
        else:
            return None

    def to_path_value(self, obj):
        """
        Convert a string or object to a path-friendly value

        :param obj: object or string value

        :return string: quoted value
        """
        if type(obj) == list:
            return ','.join(obj)
        else:
            return str(obj)

    def sanitize_for_serialization(self, obj):
        """
        Sanitize an object for Request.

        If obj is None, return None.
        If obj is str, int, float, bool, unicode, return directly.
        If obj is datetime.datetime, datetime.date convert to string in iso8601 format.
        If obj is list, santize each element in the list.
        If obj is dict, return the dict.
        If obj is swagger model, return the properties dict.
        """
        if isinstance(obj, type(None)):
            return None
        elif isinstance(obj, (int, float, bool, tuple, text_type, str)):
            return obj
        elif isinstance(obj, list):
            return [self.sanitize_for_serialization(sub_obj) for sub_obj in obj]
        elif isinstance(obj, (datetime.datetime, datetime.date)):
            return obj.isoformat()
        else:
            if isinstance(obj, dict):
                obj_dict = obj
            else:
                # Convert model obj to dict except attributes `swagger_types`, `attribute_map`
                # and attributes which value is not None.
                # Convert attribute name to json key in model definition for request.
                obj_dict = {
                    obj.attribute_map[key]: val
                    for key, val in iteritems(obj.__dict__)
                    if key not in ['swagger_types', 'attribute_map']
                    and val is not None
                }
            return {
                key: self.sanitize_for_serialization(val)
                for key, val in iteritems(obj_dict)
            }

    def deserialize(self, obj, obj_class):
        """
        Derialize a JSON string into an object.

        :param obj: string or object to be deserialized
        :param obj_class: class literal for deserialzied object, or string of class name

        :return object: deserialized object
        """
        # Have to accept obj_class as string or actual type. Type could be a
        # native Python type, or one of the model classes.

        if 'list[' in obj_class:
            match = re.match('list\[(.*)\]', obj_class)
            sub_class = match.group(1)
            return [self.deserialize(sub_obj, sub_class) for sub_obj in obj]
        if obj_class == 'str':
            return text_type(obj)
        if obj_class in ['int', 'float', 'dict', 'list', 'bool']:
            try:
                return eval(obj_class)(obj)
            except TypeError:
                return obj
        if obj_class == 'datetime':
            return self.__parse_string_to_datetime(obj)

        model_class = getattr(models, obj_class)
        return self.deserialize_model(model_class, obj)

    def deserialize_model(self, model_class, obj):
        instance = model_class()
        if obj is None or type(obj) not in [list, dict]:
            return instance
        for attr, attr_type in iteritems(instance.swagger_types):
            if instance.attribute_map[attr] not in obj:
                continue
            value = obj[instance.attribute_map[attr]]
            setattr(instance, attr, self.deserialize(value, attr_type))
        return instance

    def __parse_string_to_datetime(self, string):
        """
        Parse datetime in string to datetime.

        The string should be in iso8601 datetime format.
        """
        from dateutil.parser import parse
        return parse(string)

    def request(self, method, url, query_params=None, headers=None, post_params=None, body=None):
        """
        Perform http request using RESTClient.
        """
        if method == "GET":
            return RESTClient.GET(url, query_params=query_params, headers=headers)
        elif method == "HEAD":
            return RESTClient.HEAD(url, query_params=query_params, headers=headers)
        elif method == "POST":
            return RESTClient.POST(url, headers=headers, post_params=post_params, body=body)
        elif method == "PUT":
            return RESTClient.PUT(url, headers=headers, post_params=post_params, body=body)
        elif method == "PATCH":
            return RESTClient.PATCH(url, headers=headers, post_params=post_params, body=body)
        elif method == "DELETE":
            return RESTClient.DELETE(url, query_params=query_params, headers=headers)
        else:
            raise ValueError("http method must be `GET`, `HEAD`, `POST`, `PATCH`, `PUT` or `DELETE`")

    def prepare_post_parameters(self, post_params=None, files=None):
        params = {}

        if post_params:
            params.update(post_params)

        if files:
            for k, v in iteritems(files):
                with open(v, 'rb') as f:
                    filename = os.path.basename(f.name)
                    filedata = f.read()
                    mimetype = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
                    params[k] = tuple([filename, filedata, mimetype])

        return params

    def select_header_accept(self, accepts):
        """
        Return `Accept` based on an array of accepts provided
        """
        if not accepts:
            return

        accepts = list(map(lambda x: x.lower(), accepts))

        if 'application/json' in accepts:
            return 'application/json'
        else:
            return ', '.join(accepts)

    def select_header_content_type(self, content_types):
        """
        Return `Content-Type` baseed on an array of content_types provided
        """
        if not content_types:
            return 'application/json'

        content_types = list(map(lambda x: x.lower(), content_types))

        if 'application/json' in content_types:
            return 'application/json'
        else:
            return content_types[0]

    def update_params_for_auth(self, headers, querys, auth_settings):
        """
        Update header and query params based on authentication setting
        """
        if not auth_settings:
            return

        for auth in auth_settings:
            auth_setting = self.configuration.auth_settings().get(auth)
            headers[auth_setting['key']] = auth_setting['value']



#!/usr/bin/env python
# coding: utf-8

"""
DataApi.py
Copyright 2015 SmartBear Software

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

NOTE: This class is auto generated by the swagger code generator program. Do not edit the class manually.
"""
from __future__ import absolute_import

# python 2 and python 3 compatibility library
from six import iteritems

from . import AbstractBaseApi


class DataApi(AbstractBaseApi):

    def __init__(self, api_client=None):
        if api_client:
            self.api_client = api_client

    def public_api_data_type_list_list(
        self,
        content_type=None,
        **kwargs
    ):
        """
        Returns a list of supported data types
        The Data Types API  <br/>Returns a list of supported data types
        :return: DepotSerializer
        """
        all_params = [
        ]

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                error_message = "Got an unexpected keyword argument '%s' " \
                                "to method public_api_data_type_list_list" % key
                raise TypeError(error_message)
            params[key] = val
        del params['kwargs']

        resource_path = '/api/v1/data/'.replace('{format}', 'json')
        method = 'GET'

        # path parameters
        allowed_params = [
        ]
        path_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        path_params = dict(path_param_items)
        # query parameters
        allowed_params = [
        ]
        query_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        query_params = dict(query_param_items)
        # headers
        allowed_params = [
        ]
        header_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        header_params = dict(header_param_items)
        # form parameters
        allowed_params = [
        ]
        form_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        form_params = dict(form_param_items)
        # files
        allowed_params = [
        ]
        file_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        files = dict(file_param_items)
        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept([])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        content_types = []
        if content_type is None or content_type not in content_types:
            content_type = self.api_client.select_header_content_type(content_types)
        header_params['Content-Type'] = content_type

        response = self.api_client.call_api(
            resource_path, method, path_params, query_params, header_params,
            body=body_params, post_params=form_params, files=files,
            response='DepotSerializer')
        return response

    def public_api_data_list(
        self,
        data_type,
        content_type=None,
        **kwargs
    ):
        """
        Returns a description of a data type and its schema
        The Data API  <br/>Returns a description of a data type and its schema
        :param str data_type: The id of the data type. Refer to the data types API for available types(required)
        :return: PublicApiDataListResponse
        """
        all_params = [
            'data_type',
        ]

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                error_message = "Got an unexpected keyword argument '%s' " \
                                "to method public_api_data_list" % key
                raise TypeError(error_message)
            params[key] = val
        del params['kwargs']

        resource_path = '/api/v1/data/{data_type}/'.replace('{format}', 'json')
        method = 'GET'

        # path parameters
        allowed_params = [
            'data_type',
        ]
        path_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        path_params = dict(path_param_items)
        # query parameters
        allowed_params = [
        ]
        query_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        query_params = dict(query_param_items)
        # headers
        allowed_params = [
        ]
        header_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        header_params = dict(header_param_items)
        # form parameters
        allowed_params = [
        ]
        form_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        form_params = dict(form_param_items)
        # files
        allowed_params = [
        ]
        file_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        files = dict(file_param_items)
        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept([])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        content_types = []
        if content_type is None or content_type not in content_types:
            content_type = self.api_client.select_header_content_type(content_types)
        header_params['Content-Type'] = content_type

        response = self.api_client.call_api(
            resource_path, method, path_params, query_params, header_params,
            body=body_params, post_params=form_params, files=files,
            response='PublicApiDataListResponse')
        return response

    def public_api_data_create(
        self,
        data_type,
        body,
        content_type=None,
        **kwargs
    ):
        """
        Uploads a list of data objects
        The Data API  <br/>Uploads a list of data objects
        :param str data_type:(required)
        :param Serializer body: A list of data records.(required)
        :return: PublicApiDataCreateResponse
        """
        all_params = [
            'data_type',
            'body',
        ]

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                error_message = "Got an unexpected keyword argument '%s' " \
                                "to method public_api_data_create" % key
                raise TypeError(error_message)
            params[key] = val
        del params['kwargs']

        resource_path = '/api/v1/data/{data_type}/'.replace('{format}', 'json')
        method = 'POST'

        # path parameters
        allowed_params = [
            'data_type',
        ]
        path_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        path_params = dict(path_param_items)
        # query parameters
        allowed_params = [
        ]
        query_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        query_params = dict(query_param_items)
        # headers
        allowed_params = [
        ]
        header_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        header_params = dict(header_param_items)
        # form parameters
        allowed_params = [
        ]
        form_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        form_params = dict(form_param_items)
        # files
        allowed_params = [
        ]
        file_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        files = dict(file_param_items)
        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept([])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        content_types = ['application/json', 'text/csv', 'application/vnd.ms-excel']
        if content_type is None or content_type not in content_types:
            content_type = self.api_client.select_header_content_type(content_types)
        header_params['Content-Type'] = content_type

        response = self.api_client.call_api(
            resource_path, method, path_params, query_params, header_params,
            body=body_params, post_params=form_params, files=files,
            response='PublicApiDataCreateResponse')
        return response



from __future__ import absolute_import

# import apis into api package
from .base_api import AbstractBaseApi
from .data_api import DataApi
from .account_api import AccountApi


__all__ = [
    'AbstractBaseApi',
    'DataApi',
    'AccountApi',
]



import json
from functools import partial
import unittest
from mock import patch, MagicMock

from ambition.rest import ApiException


class RestClientObjectTest(unittest.TestCase):
    def setUp(self):
        pass

    @patch('ambition.rest.RESTClientObject.agent')
    def test_content_types(self, rest_agent):
        """
        Verify that the rest client supports the extended set of content types
        """
        request_mock = MagicMock(return_value=MagicMock(status=200, data=b''))
        rest_agent.return_value.request = request_mock
        from ..rest import RESTClient
        test_headers = {
            'Content-Type': 'application/vnd.ms-excel'
        }
        RESTClient.POST('fake_url', headers=test_headers)
        request_mock.assert_called_with(
            'POST', 'fake_url', body=None, headers=test_headers)

    @patch('ambition.rest.RESTClientObject.agent')
    def test_get_query_params(self, rest_agent):
        """
        Verify that the rest client passes along query params to transport
        """
        request_mock = MagicMock(return_value=MagicMock(status=200, data=b''))
        rest_agent.return_value.request = request_mock
        from ..rest import RESTClient
        query_params = {'foo': 'bar'}
        RESTClient.GET('fake_url', query_params=query_params)
        request_kwargs = {
            'fields': query_params,
            'headers': {
                'Content-Type': 'application/json',
            },
        }
        request_mock.assert_called_with('GET', 'fake_url', **request_kwargs)

    @patch('ambition.rest.RESTClientObject.agent')
    def test_form_params(self, rest_agent):
        """
        Verify that the rest client passes along form params to transport
        """
        request_mock = MagicMock(return_value=MagicMock(status=200, data=b''))
        rest_agent.return_value.request = request_mock
        from ..rest import RESTClient
        post_params = {'foo': 'bar'}
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        RESTClient.POST('fake_url', post_params=post_params, headers=headers)
        request_kwargs = {
            'fields': post_params,
            'headers': headers,
            'encode_multipart': False,
        }
        request_mock.assert_called_with('POST', 'fake_url', **request_kwargs)

    def test_agent_selector(self):
        """
        Verifies that the pool with the right scheme is returned
        """
        from ..rest import RESTClientObject
        client = RESTClientObject()
        http_url = 'http://www.google.com'
        self.assertIsNone(client.agent(http_url).connection_pool_kw.get('cert_reqs'))
        https_url = 'https://www.google.com'
        self.assertIsNotNone(client.agent(https_url).connection_pool_kw.get('cert_reqs'))

    @patch('ambition.rest.RESTClientObject.request')
    def test_put(self, request_mock):
        """
        Verifies that put correctly wraps the request method
        """
        from ..rest import RESTClient
        client = RESTClient()
        url = 'http://www.example.com'
        client.PUT(url)
        request_mock.assert_called_with(
            'PUT', url, headers=None, post_params=None, body=None)

    @patch('ambition.rest.RESTClientObject.request')
    def test_patch(self, request_mock):
        """
        Verifies that patch correctly wraps the request method
        """
        from ..rest import RESTClientObject
        client = RESTClientObject()
        url = 'http://www.example.com'
        client.PATCH(url)
        request_mock.assert_called_with(
            'PATCH', url, headers=None, post_params=None, body=None)

    @patch('ambition.rest.RESTClientObject.request')
    def test_head(self, request_mock):
        """
        Verifies that head correctly wraps the request method
        """
        from ..rest import RESTClient
        client = RESTClient()
        url = 'http://www.example.com'
        client.HEAD(url)
        request_mock.assert_called_with(
            'HEAD', url, headers=None, query_params=None)

    @patch('ambition.rest.RESTClientObject.request')
    def test_delete(self, request_mock):
        """
        Verifies that delete correctly wraps the request method
        """
        from ..rest import RESTClientObject
        client = RESTClientObject()
        url = 'http://www.example.com'
        client.DELETE(url)
        request_mock.assert_called_with(
            'DELETE', url, headers=None, query_params=None)

    def test_error_response(self):
        """
        Verifies that API errors result in well constructed objects
        """
        from ..rest import ApiException
        response = {
            'status': 500,
            'reason': 'just because',
            'data': b'not}{json',
            'headers': []
        }
        exception = ApiException(MagicMock(**response))
        self.assertEqual(type(str(exception)), str)

    @patch('ambition.rest.RESTClientObject.PATCH')
    def test_rest_client_patch(self, patch):
        """
        Verifies that the patch method invokes the corresponding method on the
        rest client object
        """
        from ..rest import RESTClient
        client = RESTClient()
        client.PATCH()
        patch.assert_called_with()

    @patch('ambition.rest.RESTClientObject.DELETE')
    def test_rest_client_delete(self, delete):
        """
        Verifies that the delete method invokes the corresponding method on the
        rest client object
        """
        from ..rest import RESTClient
        client = RESTClient()
        client.DELETE()
        delete.assert_called_with()

    @patch('ambition.rest.RESTClientObject.agent')
    def test_multipart_boundary_allowed(self, rest_agent):
        """
        Verifies that the API client doesn't override urllib's definition of
        the multipart form boundary
        """
        request_mock = MagicMock(return_value=MagicMock(status=200, data=b''))
        rest_agent.return_value.request = request_mock
        from ..rest import RESTClientObject
        client = RESTClientObject()
        post_params = {'foo': 'bar'}
        headers = {'Content-Type': 'multipart/form-data'}
        client.request(
            'POST', 'http://example.com/', post_params=post_params,
            headers=headers)
        request_mock.assert_called_with(
            'POST', 'http://example.com/', fields=post_params,
            encode_multipart=True, headers={}
        )

    @patch('ambition.rest.RESTClientObject.agent')
    def test_query_params_are_added_to_url(self, rest_agent):
        """
        Verify that query params are appended to the url correctly
        """
        request_mock = MagicMock(return_value=MagicMock(status=200, data=b''))
        rest_agent.return_value.request = request_mock
        from ..rest import RESTClientObject
        client = RESTClientObject()
        query_params = {'foo': 'bar'}
        client.request(
            'POST', 'http://example.com/', query_params=query_params,
            body=query_params
        )
        expected_headers = {'Content-Type': 'application/json'}
        request_mock.assert_called_with(
            'POST', 'http://example.com/?foo=bar', body=json.dumps(query_params),
            headers=expected_headers
        )

    @patch('ambition.rest.RESTClientObject.agent')
    def test_api_error_raises_exception(self, rest_agent):
        """
        Verify that an api error response will raise an exception
        """
        request_mock = MagicMock(return_value=MagicMock(status=500, data=b''))
        rest_agent.return_value.request = request_mock
        from ..rest import RESTClientObject
        client = RESTClientObject()
        query_params = {'foo': 'bar'}
        request = partial(
            client.request, 'POST', 'http://example.com/',
            query_params=query_params, body='')
        self.assertRaises(ApiException, request)



from abc import ABCMeta

from ..configuration import ApiConfiguration
from ..api_client import ApiClient


class AbstractBaseApi(object):
    """
    Base Api class which provides a factory method
    """
    __metaclass__ = ABCMeta

    @classmethod
    def create_client(cls, subdomain, api_key, scheme='http', port=None):
        """
        Factory method for creating a client for this api
        """
        hostname = '%s://%s.ambition.com' % (scheme, subdomain)
        if port is not None:
            hostname = '%s:%s' % (hostname, port)
        configuration = ApiConfiguration(hostname, api_key)
        client = ApiClient(configuration)
        return cls(client)



#!/usr/bin/env python
# coding: utf-8

"""
AccountApi.py
Copyright 2015 SmartBear Software

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

NOTE: This class is auto generated by the swagger code generator program. Do not edit the class manually.
"""
from __future__ import absolute_import

# python 2 and python 3 compatibility library
from six import iteritems

from . import AbstractBaseApi


class AccountApi(AbstractBaseApi):

    def __init__(self, api_client=None):
        if api_client:
            self.api_client = api_client

    def public_api_account_list(
        self,
        content_type=None,
        **kwargs
    ):
        """
        Return a list of accounts
        The Account API  <br/>Return a list of accounts.
        :return: AccountApiSerializer
        """
        all_params = [
        ]

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                error_message = "Got an unexpected keyword argument '%s' " \
                                "to method public_api_account_list" % key
                raise TypeError(error_message)
            params[key] = val
        del params['kwargs']

        resource_path = '/api/v1/account'.replace('{format}', 'json')
        method = 'GET'

        # path parameters
        allowed_params = [
        ]
        path_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        path_params = dict(path_param_items)
        # query parameters
        allowed_params = [
        ]
        query_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        query_params = dict(query_param_items)
        # headers
        allowed_params = [
        ]
        header_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        header_params = dict(header_param_items)
        # form parameters
        allowed_params = [
        ]
        form_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        form_params = dict(form_param_items)
        # files
        allowed_params = [
        ]
        file_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        files = dict(file_param_items)
        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept([])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        content_types = []
        if content_type is None or content_type not in content_types:
            content_type = self.api_client.select_header_content_type(content_types)
        header_params['Content-Type'] = content_type

        response = self.api_client.call_api(
            resource_path, method, path_params, query_params, header_params,
            body=body_params, post_params=form_params, files=files,
            response='AccountApiSerializer')
        return response

    def public_api_account_create(
        self,
        username,
        first_name,
        last_name,
        time_zone,
        email,
        content_type=None,
        **kwargs
    ):
        """
        Create a new account
        The Account API  <br/>Create a new account
        :param str username:(required)
        :param int superior: The id for the account&#39;s superior
        :param str first_name:(required)
        :param str last_name:(required)
        :param str time_zone:(required)
        :param bool is_manager:
        :param str email:(required)
        :param str password:
        :return: AccountApiSerializer
        """
        all_params = [
            'username',
            'superior',
            'first_name',
            'last_name',
            'time_zone',
            'is_manager',
            'email',
            'password',
        ]

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                error_message = "Got an unexpected keyword argument '%s' " \
                                "to method public_api_account_create" % key
                raise TypeError(error_message)
            params[key] = val
        del params['kwargs']

        resource_path = '/api/v1/account'.replace('{format}', 'json')
        method = 'POST'

        # path parameters
        allowed_params = [
        ]
        path_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        path_params = dict(path_param_items)
        # query parameters
        allowed_params = [
        ]
        query_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        query_params = dict(query_param_items)
        # headers
        allowed_params = [
        ]
        header_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        header_params = dict(header_param_items)
        # form parameters
        allowed_params = [
            'username',
            'superior',
            'first_name',
            'last_name',
            'time_zone',
            'is_manager',
            'email',
            'password',
        ]
        form_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        form_params = dict(form_param_items)
        # files
        allowed_params = [
        ]
        file_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        files = dict(file_param_items)
        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept([])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        content_types = []
        if content_type is None or content_type not in content_types:
            content_type = self.api_client.select_header_content_type(content_types)
        header_params['Content-Type'] = content_type

        response = self.api_client.call_api(
            resource_path, method, path_params, query_params, header_params,
            body=body_params, post_params=form_params, files=files,
            response='AccountApiSerializer')
        return response

    def public_api_account_retrieve(
        self,
        pk,
        content_type=None,
        **kwargs
    ):
        """
        Return a single account
        The Account API  <br/>Return a single account.
        :param str pk:(required)
        :return: AccountApiSerializer
        """
        all_params = [
            'pk',
        ]

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                error_message = "Got an unexpected keyword argument '%s' " \
                                "to method public_api_account_retrieve" % key
                raise TypeError(error_message)
            params[key] = val
        del params['kwargs']

        resource_path = '/api/v1/account/{pk}'.replace('{format}', 'json')
        method = 'GET'

        # path parameters
        allowed_params = [
            'pk',
        ]
        path_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        path_params = dict(path_param_items)
        # query parameters
        allowed_params = [
        ]
        query_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        query_params = dict(query_param_items)
        # headers
        allowed_params = [
        ]
        header_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        header_params = dict(header_param_items)
        # form parameters
        allowed_params = [
        ]
        form_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        form_params = dict(form_param_items)
        # files
        allowed_params = [
        ]
        file_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        files = dict(file_param_items)
        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept([])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        content_types = []
        if content_type is None or content_type not in content_types:
            content_type = self.api_client.select_header_content_type(content_types)
        header_params['Content-Type'] = content_type

        response = self.api_client.call_api(
            resource_path, method, path_params, query_params, header_params,
            body=body_params, post_params=form_params, files=files,
            response='AccountApiSerializer')
        return response

    def public_api_account_update(
        self,
        pk,
        username,
        first_name,
        last_name,
        email,
        password,
        content_type=None,
        **kwargs
    ):
        """
        The Account API
        The Account API
        :param str pk:(required)
        :param str username:(required)
        :param str superior:
        :param str first_name:(required)
        :param str last_name:(required)
        :param str time_zone:
        :param bool is_manager:
        :param str email:(required)
        :param str password:(required)
        :return: AccountApiSerializer
        """
        all_params = [
            'pk',
            'username',
            'superior',
            'first_name',
            'last_name',
            'time_zone',
            'is_manager',
            'email',
            'password',
        ]

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                error_message = "Got an unexpected keyword argument '%s' " \
                                "to method public_api_account_update" % key
                raise TypeError(error_message)
            params[key] = val
        del params['kwargs']

        resource_path = '/api/v1/account/{pk}'.replace('{format}', 'json')
        method = 'PUT'

        # path parameters
        allowed_params = [
            'pk',
        ]
        path_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        path_params = dict(path_param_items)
        # query parameters
        allowed_params = [
        ]
        query_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        query_params = dict(query_param_items)
        # headers
        allowed_params = [
        ]
        header_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        header_params = dict(header_param_items)
        # form parameters
        allowed_params = [
            'username',
            'superior',
            'first_name',
            'last_name',
            'time_zone',
            'is_manager',
            'email',
            'password',
        ]
        form_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        form_params = dict(form_param_items)
        # files
        allowed_params = [
        ]
        file_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        files = dict(file_param_items)
        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept([])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        content_types = []
        if content_type is None or content_type not in content_types:
            content_type = self.api_client.select_header_content_type(content_types)
        header_params['Content-Type'] = content_type

        response = self.api_client.call_api(
            resource_path, method, path_params, query_params, header_params,
            body=body_params, post_params=form_params, files=files,
            response='AccountApiSerializer')
        return response

    def public_api_account_destroy(
        self,
        pk,
        content_type=None,
        **kwargs
    ):
        """
        The Account API
        The Account API
        :param str pk:(required)
        :return: AccountApiSerializer
        """
        all_params = [
            'pk',
        ]

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                error_message = "Got an unexpected keyword argument '%s' " \
                                "to method public_api_account_destroy" % key
                raise TypeError(error_message)
            params[key] = val
        del params['kwargs']

        resource_path = '/api/v1/account/{pk}'.replace('{format}', 'json')
        method = 'DELETE'

        # path parameters
        allowed_params = [
            'pk',
        ]
        path_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        path_params = dict(path_param_items)
        # query parameters
        allowed_params = [
        ]
        query_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        query_params = dict(query_param_items)
        # headers
        allowed_params = [
        ]
        header_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        header_params = dict(header_param_items)
        # form parameters
        allowed_params = [
        ]
        form_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        form_params = dict(form_param_items)
        # files
        allowed_params = [
        ]
        file_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        files = dict(file_param_items)
        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept([])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        content_types = []
        if content_type is None or content_type not in content_types:
            content_type = self.api_client.select_header_content_type(content_types)
        header_params['Content-Type'] = content_type

        response = self.api_client.call_api(
            resource_path, method, path_params, query_params, header_params,
            body=body_params, post_params=form_params, files=files,
            response='AccountApiSerializer')
        return response

    def public_api_account_partial_update(
        self,
        pk,
        content_type=None,
        **kwargs
    ):
        """
        The Account API
        The Account API
        :param str pk:(required)
        :param str username:
        :param str superior:
        :param str first_name:
        :param str last_name:
        :param str time_zone:
        :param bool is_manager:
        :param str email:
        :param str password:
        :return: AccountApiSerializer
        """
        all_params = [
            'pk',
            'username',
            'superior',
            'first_name',
            'last_name',
            'time_zone',
            'is_manager',
            'email',
            'password',
        ]

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                error_message = "Got an unexpected keyword argument '%s' " \
                                "to method public_api_account_partial_update" % key
                raise TypeError(error_message)
            params[key] = val
        del params['kwargs']

        resource_path = '/api/v1/account/{pk}'.replace('{format}', 'json')
        method = 'PATCH'

        # path parameters
        allowed_params = [
            'pk',
        ]
        path_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        path_params = dict(path_param_items)
        # query parameters
        allowed_params = [
        ]
        query_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        query_params = dict(query_param_items)
        # headers
        allowed_params = [
        ]
        header_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        header_params = dict(header_param_items)
        # form parameters
        allowed_params = [
            'username',
            'superior',
            'first_name',
            'last_name',
            'time_zone',
            'is_manager',
            'email',
            'password',
        ]
        form_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        form_params = dict(form_param_items)
        # files
        allowed_params = [
        ]
        file_param_items = [
            item
            for item in params.items() if item[0] in allowed_params
        ]
        files = dict(file_param_items)
        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept([])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        content_types = []
        if content_type is None or content_type not in content_types:
            content_type = self.api_client.select_header_content_type(content_types)
        header_params['Content-Type'] = content_type

        response = self.api_client.call_api(
            resource_path, method, path_params, query_params, header_params,
            body=body_params, post_params=form_params, files=files,
            response='AccountApiSerializer')
        return response



# import multiprocessing to avoid this bug (http://bugs.python.org/issue15881#msg170215)
import multiprocessing
assert multiprocessing
import re
from setuptools import setup, find_packages


def get_version():
    """
    Extracts the version number from the version.py file.
    """
    VERSION_FILE = 'ambition/version.py'
    mo = re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]', open(VERSION_FILE, 'rt').read(), re.M)
    if mo:
        return mo.group(1)
    else:
        raise RuntimeError('Unable to find version string in {0}.'.format(VERSION_FILE))

# To install the library, open a Terminal shell, then run this
# file by typing:
#
# python setup.py install
#
# You need to have the setuptools module installed.
# Try reading the setuptools documentation:
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["urllib3 >= 1.10", "six >= 1.9", "certifi", "python-dateutil==2.4.2"]

setup(
    name="ambition",
    version=get_version(),
    description='',
    long_description=open('README.rst').read(),
    url='https://github.com/ambitioninc/ambition',
    author='Ryan Bales',
    author_email='opensource@ambition.com',
    install_requires=REQUIRES,
    packages=find_packages(),
    keywords='',
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    license='MIT',
    include_package_data=True,
    test_suite='nose.collector',
    tests_require=[
        'coverage>=3.7.1',
        'flake8>=2.2.0',
        'mock>=1.0.1',
        'nose>=1.3.0',
    ],
    zip_safe=False,
)



from __future__ import absolute_import

# import models into model package
from .account_api_serializer import AccountApiSerializer
from .public_api_data_create_response import PublicApiDataCreateResponse
from .write_account_api_serializer import WriteAccountApiSerializer
from .public_api_data_list_response import PublicApiDataListResponse
from .write_serializer import WriteSerializer
from .serializer import Serializer
from .depot_serializer import DepotSerializer
from .write_depot_serializer import WriteDepotSerializer


__all__ = [
    'AccountApiSerializer',
    'PublicApiDataCreateResponse',
    'WriteAccountApiSerializer',
    'PublicApiDataListResponse',
    'WriteSerializer',
    'Serializer',
    'DepotSerializer',
    'WriteDepotSerializer',
]



# coding: utf-8

"""
Credit: this file (rest.py) is modified based on rest.py in Dropbox Python SDK:
https://www.dropbox.com/developers/core/sdks/python
"""

import io
import json
import ssl
import certifi
import six

try:
    import urllib3
except ImportError:
    raise ImportError('Swagger python client requires urllib3.')

try:
    # for python3
    from urllib.parse import urlencode
except ImportError:
    # for python2
    from urllib import urlencode


class RESTResponse(io.IOBase):

    def __init__(self, resp):
        self.urllib3_response = resp
        self.status = resp.status
        self.reason = resp.reason
        self.data = resp.data

    def getheaders(self):
        """
        Returns a dictionary of the response headers.
        """
        return self.urllib3_response.getheaders()


class RESTClientObject(object):

    def __init__(self, pools_size=4):
        # http pool manager
        self.pool_manager = urllib3.PoolManager(
            num_pools=pools_size
        )

        # https pool manager
        # certificates validated using Mozilla’s root certificates
        self.ssl_pool_manager = urllib3.PoolManager(
            num_pools=pools_size,
            cert_reqs=ssl.CERT_REQUIRED,
            ca_certs=certifi.where()
        )

    def agent(self, url):
        """
        Return proper pool manager for the http\https schemes.
        """
        url = urllib3.util.url.parse_url(url)
        scheme = url.scheme
        if scheme == 'https':
            return self.ssl_pool_manager
        else:
            return self.pool_manager

    def request(self, method, url, query_params=None, headers=None,
                body=None, post_params=None):
        """
        :param method: http request method
        :param url: http request url
        :param query_params: query parameters in the url
        :param headers: http request headers
        :param body: request json body, for `application/json`
        :param post_params: request post parameters, `application/x-www-form-urlencode`
                            and `multipart/form-data`
        """
        method = method.upper()
        assert method in ['GET', 'HEAD', 'DELETE', 'POST', 'PUT', 'PATCH']

        if post_params and body:
            raise ValueError("body parameter cannot be used with post_params parameter.")

        post_params = post_params or {}
        headers = headers or {}

        if 'Content-Type' not in headers:
            headers['Content-Type'] = 'application/json'

        # For `POST`, `PUT`, `PATCH`
        if method in ['POST', 'PUT', 'PATCH']:
            if query_params:
                url += '?' + urlencode(query_params)
            if headers['Content-Type'] == 'application/json':
                r = self.agent(url).request(method, url,
                                            body=json.dumps(body),
                                            headers=headers)
            elif headers['Content-Type'] == 'application/x-www-form-urlencoded':
                r = self.agent(url).request(method, url,
                                            fields=post_params,
                                            encode_multipart=False,
                                            headers=headers)
            elif headers['Content-Type'] == 'multipart/form-data':
                # must del headers['Content-Type'], or the correct Content-Type
                # which generated by urllib3 will be overwritten.
                del headers['Content-Type']
                r = self.agent(url).request(method, url,
                                            fields=post_params,
                                            encode_multipart=True,
                                            headers=headers)
            else:
                # assumes that the body has already been serialized
                r = self.agent(url).request(method, url,
                                            body=body,
                                            headers=headers)
        # For `GET`, `HEAD`, `DELETE`
        else:
            r = self.agent(url).request(method, url,
                                        fields=query_params,
                                        headers=headers)
        r = RESTResponse(r)

        if r.status not in range(200, 206):
            raise ApiException(r)

        return self.process_response(r)

    def process_response(self, response):
        data = six.text_type(response.data)
        try:
            resp = json.loads(data)
        except ValueError:
            resp = data

        return resp

    def GET(self, url, headers=None, query_params=None):
        return self.request("GET", url, headers=headers, query_params=query_params)

    def HEAD(self, url, headers=None, query_params=None):
        return self.request("HEAD", url, headers=headers, query_params=query_params)

    def DELETE(self, url, headers=None, query_params=None):
        return self.request("DELETE", url, headers=headers, query_params=query_params)

    def POST(self, url, headers=None, post_params=None, body=None):
        return self.request("POST", url, headers=headers, post_params=post_params, body=body)

    def PUT(self, url, headers=None, post_params=None, body=None):
        return self.request("PUT", url, headers=headers, post_params=post_params, body=body)

    def PATCH(self, url, headers=None, post_params=None, body=None):
        return self.request("PATCH", url, headers=headers, post_params=post_params, body=body)


class ApiException(Exception):
    """
    Non-2xx HTTP response
    """

    def __init__(self, http_resp):
        self.status = http_resp.status
        self.reason = http_resp.reason
        self.body = http_resp.data
        self.headers = http_resp.getheaders()

        data = six.text_type(self.body)

        try:
            self.body = json.loads(data)
        except ValueError:
            self.body = data

    def __str__(self):
        """
        Custom error response messages
        """
        return "({0})\n"\
            "Reason: {1}\n"\
            "HTTP response headers: {2}\n"\
            "HTTP response body: {3}\n".\
            format(self.status, self.reason, self.headers, self.body)


class RESTClient(object):
    """
    A class with all class methods to perform JSON requests.
    """

    IMPL = RESTClientObject()

    @classmethod
    def GET(cls, *n, **kw):
        """
        Perform a GET request using `RESTClient.request()`.
        """
        return cls.IMPL.GET(*n, **kw)

    @classmethod
    def HEAD(cls, *n, **kw):
        """
        Perform a HEAD request using `RESTClient.request()`.
        """
        return cls.IMPL.HEAD(*n, **kw)

    @classmethod
    def POST(cls, *n, **kw):
        """
        Perform a POST request using `RESTClient.request()`
        """
        return cls.IMPL.POST(*n, **kw)

    @classmethod
    def PUT(cls, *n, **kw):
        """
        Perform a PUT request using `RESTClient.request()`
        """
        return cls.IMPL.PUT(*n, **kw)

    @classmethod
    def PATCH(cls, *n, **kw):
        """
        Perform a PATCH request using `RESTClient.request()`
        """
        return cls.IMPL.PATCH(*n, **kw)

    @classmethod
    def DELETE(cls, *n, **kw):
        """
        Perform a DELETE request using `RESTClient.request()`
        """
        return cls.IMPL.DELETE(*n, **kw)



from __future__ import absolute_import

# import models into sdk package
from .models.account_api_serializer import AccountApiSerializer
from .models.public_api_data_create_response import PublicApiDataCreateResponse
from .models.write_account_api_serializer import WriteAccountApiSerializer
from .models.public_api_data_list_response import PublicApiDataListResponse
from .models.write_serializer import WriteSerializer
from .models.serializer import Serializer
from .models.depot_serializer import DepotSerializer
from .models.write_depot_serializer import WriteDepotSerializer

# import apis into sdk package
from .apis.data_api import DataApi
from .apis.account_api import AccountApi

# import ApiClient
from .api_client import ApiClient

__all__ = [
    'ApiClient',
    'AccountApiSerializer',
    'PublicApiDataCreateResponse',
    'WriteAccountApiSerializer',
    'PublicApiDataListResponse',
    'WriteSerializer',
    'Serializer',
    'DepotSerializer',
    'WriteDepotSerializer',
    'DataApi',
    'AccountApi',
]



import os
from functools import partial
import datetime
import unittest

from mock import patch

from ..api_client import ApiClient
from ..configuration import ApiConfiguration
from .. import models

test_dict = {
    'name': 'Test Name',
    'display_name': 'Test Display Name',
    'data_format': 'Test Format',
}


class TestModel(object):
    def __init__(self):
        self.swagger_types = {
            'display_name': 'str',
            'name': 'str',
            'data_format': 'str',
        }
        self.attribute_map = {
            'display_name': 'display_name',
            'name': 'name',
            'data_format': 'data_format',
        }
        self.display_name = None
        self.name = None
        self.data_format = None
        self.some_other_attribute = None


class ApiClientTest(unittest.TestCase):
    def setUp(self):
        host = 'http://example.com'
        api_key = 'keyboardcat'
        configuration = ApiConfiguration(host, api_key)
        self.client = ApiClient(configuration=configuration)
        self.base_expected_headers = {
            'Authorization': 'Token keyboardcat',
            'User-Agent': 'Python-Swagger',
        }

    def test_sanitization_for_serialization(self):
        """
        Verify that data are normalized
        """
        model = TestModel()
        for key in test_dict.keys():
            setattr(model, key, test_dict[key])
        sanitized_model = self.client.sanitize_for_serialization(model)
        self.assertEqual(sanitized_model, test_dict)

    def test_deserialization(self):
        obj = [{'foo': 'bar'}, {'baz': 'qux'}]
        deserialized = self.client.deserialize(obj, 'list[dict]')
        self.assertEqual(obj, deserialized)
        obj = 1
        deserialized = self.client.deserialize(obj, 'dict')
        self.assertEqual(deserialized, obj)
        # deserialize model from dict that doesn't have all model attributes
        models.TestModel = TestModel
        obj = {'name': 'some name'}
        deserialized = self.client.deserialize(obj, 'TestModel')
        self.assertIsNone(deserialized.display_name)
        self.assertIsNone(deserialized.data_format)
        # deserialize datetimes
        now = datetime.datetime.now()
        deserialized = self.client.deserialize(now.isoformat(), 'datetime')
        self.assertEqual(now, deserialized)

    @patch('ambition.api_client.RESTClient.GET')
    @patch('ambition.api_client.RESTClient.HEAD')
    @patch('ambition.api_client.RESTClient.POST')
    @patch('ambition.api_client.RESTClient.PATCH')
    @patch('ambition.api_client.RESTClient.PUT')
    @patch('ambition.api_client.RESTClient.DELETE')
    def test_request_method(self, delete, put, patch, post, head, get):
        """
        Verify that the correct client method is called with the right kwargs
        """
        query_params = {'query': 'query_param'}
        post_params = {'post': 'post_param'}
        body = 'body'
        self.client.request(
            'GET', 'some_url', query_params=query_params, body=body,
            post_params=post_params, headers=self.base_expected_headers)
        self.client.request(
            'HEAD', 'some_url', query_params=query_params, body=body,
            post_params=post_params, headers=self.base_expected_headers)
        self.client.request(
            'POST', 'some_url', query_params=query_params, body=body,
            post_params=post_params, headers=self.base_expected_headers)
        self.client.request(
            'PATCH', 'some_url', query_params=query_params, body=body,
            post_params=post_params, headers=self.base_expected_headers)
        self.client.request(
            'PUT', 'some_url', query_params=query_params, body=body,
            post_params=post_params, headers=self.base_expected_headers)
        self.client.request(
            'DELETE', 'some_url', query_params=query_params, body=body,
            post_params=post_params, headers=self.base_expected_headers)
        delete.assert_called_with(
            'some_url', query_params=query_params,
            headers=self.base_expected_headers)
        put.assert_called_with(
            'some_url', post_params=post_params, body=body,
            headers=self.base_expected_headers)
        patch.assert_called_with(
            'some_url', post_params=post_params, body=body,
            headers=self.base_expected_headers)
        post.assert_called_with(
            'some_url', post_params=post_params, body=body,
            headers=self.base_expected_headers)
        head.assert_called_with(
            'some_url', query_params=query_params,
            headers=self.base_expected_headers)
        get.assert_called_with(
            'some_url', query_params=query_params,
            headers=self.base_expected_headers)
        n = ['NOT_A_METHOD', 'some_url']
        self.assertRaises(ValueError, partial(self.client.request, *n))

    def test_files(self):
        """
        Verifies that the files are included in post params
        """
        file_path = os.path.abspath(__file__)
        files = {
            'this_file': file_path
        }
        post_params = self.client.prepare_post_parameters(files=files)
        self.assertIn('this_file', post_params)

    def test_select_accepts(self):
        """
        Verifies that the accept header is correctly selected (or not)
        from a list
        """
        self.assertIsNone(self.client.select_header_accept([]))
        accepts = ['application/vnd.ms-excel', 'application/json']
        self.assertEqual('application/json', self.client.select_header_accept(accepts))
        accepts = ['application/vnd.ms-excel', 'text/csv']
        self.assertEqual(', '.join(accepts), self.client.select_header_accept(accepts))

    def test_select_content_type(self):
        """
        Verifies that the content type header is correctly selected
        """
        self.assertEqual('application/json', self.client.select_header_content_type([]))
        content_types = ['application/vnd.ms-excel', 'application/json']
        self.assertEqual('application/json', self.client.select_header_content_type(content_types))
        content_types = ['application/vnd.ms-excel', 'text/csv']
        self.assertEqual('application/vnd.ms-excel', self.client.select_header_content_type(content_types))

    @patch('ambition.api_client.models')
    @patch('ambition.api_client.RESTClient.GET')
    def test_deserialization_single_model(self, rest_get, models):
        """
        Verify that api responses are cast as the right model type
        """
        rest_get.return_value = test_dict
        models.TestModel = TestModel
        model = self.client.call_api('/fake', 'GET', response='TestModel')
        self.assertIsInstance(model, TestModel)
        self.assertEqual(model.display_name, test_dict.get('display_name'))
        self.assertEqual(model.name, test_dict.get('name'))
        self.assertEqual(model.data_format, test_dict.get('data_format'))

    @patch('ambition.api_client.models')
    @patch('ambition.api_client.RESTClient.GET')
    def test_deserialization_multiple_models(self, rest_get, models):
        """
        Verify that list api responses are model iterators
        """
        serialized_response = [test_dict, test_dict]
        rest_get.return_value = serialized_response
        models.TestModel = TestModel
        response = self.client.call_api('/fake', 'GET', response='TestModel')
        self.assertEqual(len(list(response)), 2)
        for model in response:
            self.assertIsInstance(model, TestModel)

    @patch('ambition.api_client.ApiClient.request')
    def test_path_params(self, request_mock):
        """
        Verify that path parameters are constructed properly
        """
        path_params = {
            'foo': 'f',
            'bar': 'b',
        }
        self.client.call_api('/{foo}/{bar}/', 'GET', path_params=path_params)
        expected_url = 'http://example.com/f/b/'
        request_mock.assert_called_with(
            'GET', expected_url, body=None,
            headers=self.base_expected_headers,
            post_params=None, query_params=None)

    @patch('ambition.api_client.ApiClient.request')
    def test_query_params(self, request_mock):
        """
        Verify that query parameters are normalized
        """
        today = datetime.datetime.now().date()
        query_params = {
            'today': today,
            'users': ['Marty McFly', 'H. G. Wells'],
            'none_thing': None,
        }
        self.client.call_api('/stuff/', 'GET', query_params=query_params)
        expected_query_params = {
            'today': datetime.datetime.now().date().isoformat(),
            'users': 'Marty McFly,H. G. Wells',
            'none_thing': 'None',
        }
        request_mock.assert_called_with(
            'GET', 'http://example.com/stuff/', body=None,
            headers=self.base_expected_headers,
            post_params=None, query_params=expected_query_params)

    @patch('ambition.api_client.ApiClient.request')
    def test_post_params(self, request_mock):
        """
        Verify that post parameters are normalized
        """
        today = datetime.datetime.now().date()
        post_params = {
            'today': today,
        }
        self.client.call_api('/stuff/', 'POST', post_params=post_params)
        expected_post_params = {
            'today': datetime.datetime.now().date().isoformat()
        }
        request_mock.assert_called_with(
            'POST', 'http://example.com/stuff/', body=None,
            headers=self.base_expected_headers,
            post_params=expected_post_params, query_params=None)

    @patch('ambition.api_client.ApiClient.request')
    def test_body_normalization(self, request_mock):
        """
        Verify that body is normalized
        """
        today = datetime.datetime.now().date()
        body = today
        self.client.call_api('/stuff/', 'POST', body=body)
        request_mock.assert_called_with(
            'POST', 'http://example.com/stuff/', body=today.isoformat(),
            headers=self.base_expected_headers,
            post_params=None, query_params=None)

    def test_update_params_for_auth(self):
        """
        Verify that authentication is defined correctly
        """
        auth_settings = None
        headers = {}
        query_params = {}
        self.client.update_params_for_auth(headers, query_params, auth_settings)
        # confirm that neither dict was modified
        self.assertEqual({}, headers)
        self.assertEqual({}, query_params)

    def test_user_agent(self):
        """
        Verifies that clients are being constructed with user agent
        """
        self.assertEqual('Python-Swagger', self.client.user_agent)

    def test_deserialize_model_gracefully_handles_bad_input(self):
        """
        Verifies that we won't try to enumerate an object not of list/dict type
        when trying to cast it to a model type
        """
        from ambition.models import PublicApiDataListResponse
        model = self.client.deserialize_model(PublicApiDataListResponse, None)
        self.assertIsInstance(model, PublicApiDataListResponse)
        for attribute in model.attribute_map:
            self.assertIsNone(getattr(model, attribute))

    def test_deserialize_datetimes(self):
        """
        Verifies that datetimes are deserialized
        """
        now = datetime.datetime.now()
        now_deserialized = self.client.deserialize(now.isoformat(), 'datetime')
        self.assertEqual(now, now_deserialized)



from __future__ import absolute_import


class ApiConfiguration(object):
    """
    API Configuration Class
    """
    # Base url
    host = None

    # Authentication settings
    _api_key = None
    _api_key_prefix = None

    def __init__(self, host, api_key, api_key_prefix='Token'):
        self.host = host
        self._api_key = api_key
        self._api_key_prefix = api_key_prefix

    @property
    def api_key(self):
        return self._api_key_prefix + ' ' + self._api_key

    def auth_settings(self):
        return {
            'default': {
                'type': 'api_key',
                'in': 'header',
                'key': 'Authorization',
                'value': self.api_key
            },
        }
